--- old/usr/src/man/man2/unlink.2	2016-12-05 21:50:22.574954942 +0000
+++ new/usr/src/man/man2/unlink.2	2016-12-05 21:50:22.587593018 +0000
@@ -2,6 +2,7 @@
 .\" Copyright (c) 2007, Sun Microsystems, Inc.  All Rights Reserved.
 .\" Copyright 1989 AT&T
 .\" Portions Copyright (c) 1992, X/Open Company Limited  All Rights Reserved.
+.\" Copyright 2016 RackTop Systems.
 .\" Sun Microsystems, Inc. gratefully acknowledges The Open Group for permission to reproduce portions of its copyrighted documentation. Original documentation from The Open Group can be obtained online at
 .\" http://www.opengroup.org/bookstore/.
 .\" The Institute of Electrical and Electronics Engineers and The Open Group, have given us permission to reprint portions of their documentation. In the following statement, the phrase "this text" refers to portions of the system documentation. Portions of this text are reprinted and reproduced in electronic form in the Sun OS Reference Manual, from IEEE Std 1003.1, 2004 Edition, Standard for Information Technology -- Portable Operating System Interface (POSIX), The Open Group Base Specifications Issue 6, Copyright (C) 2001-2004 by the Institute of Electrical and Electronics Engineers, Inc and The Open Group. In the event of any discrepancy between these versions and the original IEEE and The Open Group Standard, the original IEEE and The Open Group Standard is the referee document. The original Standard can be obtained online at http://www.opengroup.org/unix/online.html.
@@ -9,7 +10,7 @@
 .\" The contents of this file are subject to the terms of the Common Development and Distribution License (the "License").  You may not use this file except in compliance with the License.
 .\" You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE or http://www.opensolaris.org/os/licensing.  See the License for the specific language governing permissions and limitations under the License.
 .\" When distributing Covered Code, include this CDDL HEADER in each file and include the License file at usr/src/OPENSOLARIS.LICENSE.  If applicable, add the following below this CDDL HEADER, with the fields enclosed by brackets "[]" replaced with your own identifying information: Portions Copyright [yyyy] [name of copyright owner]
-.TH UNLINK 2 "May 18, 2007"
+.TH UNLINK 2 "Dec 7, 2016"
 .SH NAME
 unlink, unlinkat \- remove directory entry
 .SH SYNOPSIS
@@ -119,6 +120,15 @@
 .RE
 
 .sp
+.ne 2
+.na
+\fB\fBEEXIST\fR\fR
+.ad
+.RS 16n
+The entry to be unlinked is a directory which is not empty.
+.RE
+
+.sp
 .ne 2
 .na
 \fB\fBEFAULT\fR\fR
--- old/usr/src/uts/common/fs/tmpfs/tmp_dir.c	2016-12-05 21:50:23.031978149 +0000
+++ new/usr/src/uts/common/fs/tmpfs/tmp_dir.c	2016-12-05 21:50:23.044513391 +0000
@@ -21,10 +21,10 @@
 /*
  * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
  * Use is subject to license terms.
+ *
+ * Copyright 2016 RackTop Systems.
  */
 
-#pragma ident	"%Z%%M%	%I%	%E% SMI"
-
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/sysmacros.h>
@@ -487,6 +487,13 @@
 	if (tp != tnp)
 		return (ENOENT);
 
+	/*
+	 * If we are unlinking a directory which isn't empty (it has
+	 * more than the minumum '.' and '..' entries) return EEXIST.
+	 */
+	if (tp->tn_type == VDIR && tp->tn_dirents > 2)
+		return (EEXIST);
+
 	tmpfs_hash_out(tpdp);
 
 	/*
@@ -527,7 +534,7 @@
 
 	ASSERT(tp->tn_nlink > 0);
 	DECR_COUNT(&tp->tn_nlink, &tp->tn_tlock);
-	if (op == DR_RMDIR && tp->tn_type == VDIR) {
+	if (tp->tn_type == VDIR) {
 		tdirtrunc(tp);
 		ASSERT(tp->tn_nlink == 0);
 	}
--- old/usr/src/uts/common/fs/tmpfs/tmp_vnops.c	2016-12-05 21:50:23.324431511 +0000
+++ new/usr/src/uts/common/fs/tmpfs/tmp_vnops.c	2016-12-05 21:50:23.335284232 +0000
@@ -1105,8 +1105,7 @@
 
 	if (tp->tn_type != VDIR ||
 	    (error = secpolicy_fs_linkdir(cred, dvp->v_vfsp)) == 0)
-		error = tdirdelete(parent, tp, nm, tp->tn_type == VDIR ?
-		    DR_RMDIR : DR_REMOVE, cred);
+		error = tdirdelete(parent, tp, nm, DR_REMOVE, cred);
 
 	rw_exit(&tp->tn_rwlock);
 	rw_exit(&parent->tn_rwlock);
@@ -1428,20 +1427,6 @@
 		goto done;
 	}
 
-	/*
-	 * Check for an empty directory
-	 * i.e. only includes entries for "." and ".."
-	 */
-	if (self->tn_dirents > 2) {
-		error = EEXIST;		/* SIGH should be ENOTEMPTY */
-		/*
-		 * Update atime because checking tn_dirents is logically
-		 * equivalent to reading the directory
-		 */
-		gethrestime(&self->tn_atime);
-		goto done;
-	}
-
 	error = tdirdelete(parent, self, nm, DR_RMDIR, cred);
 done:
 	vn_vfsunlock(vp);
